# generated by datamodel-codegen:
#   filename:  https://raw.githubusercontent.com/HumanBrainProject/openMINDS/3fa86f956b407b2debf47c2e1b6314e37579c707/v3/core/v4/data/license.schema.json

from typing import List, Optional

from pydantic import Field
from siibra.openminds.base import SiibraBaseModel


class Model(SiibraBaseModel):
    id: str = Field(..., alias='@id', description='Metadata node identifier.')
    type: str = Field(..., alias='@type')
    full_name: str = Field(
        ...,
        alias='fullName',
        description='Whole, non-abbreviated name of something or somebody.',
        title='fullName',
    )
    legal_code: str = Field(
        ...,
        alias='legalCode',
        description='Type of legislation that claims to cover the law system (complete or parts) as it existed at the time the code was enacted.',
        title='legalCode',
    )
    short_name: str = Field(
        ...,
        alias='shortName',
        description='Shortened or fully abbreviated name of something or somebody.',
        title='shortName',
    )
    webpage: Optional[List[str]] = Field(
        None,
        alias='webpage',
        description='Hypertext document (block of information) found on the World Wide Web.',
        min_items=1,
        title='webpage',
    )
