# generated by datamodel-codegen:
#   filename:  https://raw.githubusercontent.com/HumanBrainProject/openMINDS/3fa86f956b407b2debf47c2e1b6314e37579c707/v3/core/v4/data/fileRepositoryStructure.schema.json

from typing import Any, Optional

from pydantic import Field
from siibra.openminds.base import SiibraBaseModel


class FilePathPattern(SiibraBaseModel):
    grouping_type: Any = Field(..., alias='groupingType', title='groupingType')
    regex: str = Field(..., title='regex')


class Model(SiibraBaseModel):
    id: str = Field(..., alias='@id', description='Metadata node identifier.')
    type: str = Field(..., alias='@type')
    file_path_pattern: 'FilePathPattern' = Field(
        ..., alias='filePathPattern'
    )
    lookup_label: Optional[str] = Field(
        None,
        alias='lookupLabel',
        title='lookupLabel',
    )
