# generated by datamodel-codegen:
#   filename:  https://raw.githubusercontent.com/HumanBrainProject/openMINDS/3fa86f956b407b2debf47c2e1b6314e37579c707/v3/core/v4/actors/affiliation.schema.json

from datetime import date
from typing import Any, Dict, Optional

from pydantic import Field
from siibra.openminds.base import SiibraBaseModel


class Model(SiibraBaseModel):
    id: str = Field(..., alias='@id', description='Metadata node identifier.')
    type: str = Field(..., alias='@type')
    end_date: Optional[date] = Field(
        None,
        alias='endDate',
        description='Date in the Gregorian calendar at which something terminates in time.',
        title='endDate',
    )
    organization: Dict[str, Any] = Field(
        ...,
        alias='organization',
        description='Legally accountable, administrative and functional structure.',
        title='organization',
    )
    start_date: Optional[date] = Field(
        None,
        alias='startDate',
        description='Date in the Gregorian calendar at which something begins in time',
        title='startDate',
    )
