# generated by datamodel-codegen:
#   filename:  https://raw.githubusercontent.com/HumanBrainProject/openMINDS/3fa86f956b407b2debf47c2e1b6314e37579c707/v3/core/v4/data/contentType.schema.json

from typing import Any, List, Optional

from pydantic import Field
from siibra.openminds.base import SiibraBaseModel


class Model(SiibraBaseModel):
    id: str = Field(..., alias='@id', description='Metadata node identifier.')
    type: str = Field(..., alias='@type')
    data_type: Optional[List[Any]] = Field(
        None,
        alias='dataType',
        min_items=1,
        title='dataType',
    )
    description: Optional[str] = Field(
        None,
        alias='description',
        description='Longer statement or account giving the characteristics of someone or something.',
        title='description',
    )
    file_extension: Optional[List[str]] = Field(
        None,
        alias='fileExtension',
        description='String of characters attached as suffix to the names of files of a particular format.',
        min_items=1,
        title='fileExtension',
    )
    name: str = Field(
        ...,
        alias='name',
        description='Word or phrase that constitutes the distinctive designation of a being or thing.',
        title='name',
    )
    related_media_type: Optional[str] = Field(
        None,
        alias='relatedMediaType',
        description='Reference to an official two-part identifier for file formats and format contents.',
        title='relatedMediaType',
    )
    specification: Optional[str] = Field(
        None,
        alias='specification',
        description='Detailed and precise presentation of, or proposal for something.',
        title='specification',
    )
    synonym: Optional[List[str]] = Field(
        None,
        alias='synonym',
        description='Words or expressions used in the same language that have the same or nearly the same meaning in some or all senses.',
        min_items=1,
        title='synonym',
    )
