# generated by datamodel-codegen:
#   filename:  https://raw.githubusercontent.com/HumanBrainProject/openMINDS/3fa86f956b407b2debf47c2e1b6314e37579c707/v3/core/v4/actors/contribution.schema.json

from typing import Any, Dict, List

from pydantic import Field
from siibra.openminds.base import SiibraBaseModel


class Model(SiibraBaseModel):
    id: str = Field(..., alias='@id', description='Metadata node identifier.')
    type: str = Field(..., alias='@type')
    contribution_type: List[Any] = Field(
        ...,
        alias='contributionType',
        description='Distinct class of what was given or supplied as a part or share.',
        min_items=1,
        title='contributionType',
    )
    contributor: Dict[str, Any] = Field(
        ...,
        alias='contributor',
        description='Legal person that gave or supplied something as a part or share.',
        title='contributor',
    )
